package task.db2;

import com.google.java.contract.Ensures;
import com.google.java.contract.Invariant;
import com.google.java.contract.Requires;
import com.google.java.contract.ThrowEnsures;

@Invariant("ntload < 4")
public abstract class Database {

	@Requires({
		"name != null",
		"passwd != null",
		"isRegistered(name) == false"
	})
	@Ensures("isRegistered(old(name)) == true")
	public abstract void register(String name, String passwd);
	
	@Requires({
		"name != null",
		"passwd != null",
		"isRegistered(name) == true",
		"isLoggedin(name) == false",
		"isPasswordCorrect(name,passwd) == true",
	})
	@Ensures({
		"isLoggedin(old(name)) == true",
		"ntload == old(ntload) + 1"
	})
	public abstract void login(String name, String passwd);
	
	@Requires({
		"name != null",
		"isLoggedin(name) == true"
	})
	@Ensures({
		"isLoggedin(old(name)) == false",
		"ntload == old(ntload) - 1"
	})
	public abstract void logout(String name);
	
	@Requires({
		"name != null",
		"isLoggedin(name) == true",
		"isDataAvailable(key) == true"
	})
	public abstract Integer readData(String name, Integer key);
	
	@Requires({
		"name != null",
		"isLoggedin(name) == true",
		"isDataAvailable(key) == false"
	})
	public abstract void addData(String name, Integer key, Integer value);
	
	protected abstract boolean isRegistered(String name);
	protected abstract boolean isLoggedin(String name);
	protected abstract boolean isPasswordCorrect(String name, String passwd);
	protected abstract boolean isDataAvailable(Integer key);
	
	protected abstract boolean isValidName(String name);
	protected abstract boolean isValidPasswd(String passwd);
	
	protected static int ntload = 0;
	
	public static void init() {
		ntload = 0;
	}
	
}
