package task.db2;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.google.java.contract.Ensures;
import com.google.java.contract.Invariant;
import com.google.java.contract.Requires;
import com.google.java.contract.ThrowEnsures;


public class SecureDB extends Database {
	
	public SecureDB(boolean enablelog) {
		
		data = new HashMap<ArrayList<Byte>,ArrayList<Byte>>();
		users = new HashMap<ArrayList<Byte>,ArrayList<Byte>>();
		loggedin = new ArrayList<ArrayList<Byte>>();
		
		e = new Encryption();
		logEnabled = enablelog;
	}

	@Override
	@Requires({
		"!name.isEmpty()",
		"!passwd.isEmpty()"
	})
	public void register(String name, String passwd) {
		// TODO Auto-generated method stub
		users.put(e.encrypt(name), e.encrypt(passwd));
	}

	@Override
	@Ensures("dbload == old(dbload) + 1")
	@ThrowEnsures({
		"Exception",
		"isLoggedin(name) == false"
	})
	public void login(String name, String passwd){
		// TODO Auto-generated method stub

		loggedin.add(e.encrypt(name));
		ntload++;
		dbload++;
		
		try {
			log("User: " + name + " has logged in.");
		} catch (NullPointerException ex) {
			loggedin.remove(e.encrypt(name));
			throw ex;
		}
	}

	@Override
	@Ensures("dbload == old(dbload) - 1")
	public void logout(String name) {
		// TODO Auto-generated method stub
		loggedin.remove(e.encrypt(name));
		ntload--;
		dbload--;
	}

	@Override
	@Ensures("result == (Integer) e.decrypt(data.get(e.encrypt(old(key))))")
	public Integer readData(String name, Integer key) {
		// TODO Auto-generated method stub
		return (Integer) e.decrypt(data.get(e.encrypt(key)));
	}

	@Override
	@Ensures("data.get(e.encrypt(old(key))).equals(e.encrypt(old(value)))")
	public void addData(String name, Integer key, Integer value) {
		// TODO Auto-generated method stub
		data.put(e.encrypt(key), e.encrypt(value));
	}

	@Override
	protected boolean isRegistered(String name) {
		// TODO Auto-generated method stub
		return users.containsKey(e.encrypt(name));
	}

	@Override
	protected boolean isLoggedin(String name) {
		// TODO Auto-generated method stub
		return loggedin.contains(e.encrypt(name));
	}

	@Override
	protected boolean isPasswordCorrect(String name, String passwd) {
		// TODO Auto-generated method stub
		return users.get(e.encrypt(name)).equals(e.encrypt(passwd));
	}

	@Override
	protected boolean isDataAvailable(Integer key) {
		// TODO Auto-generated method stub
		return data.containsKey(e.encrypt(key));
	}


	@Override
	protected boolean isValidName(String name) {
		// TODO Auto-generated method stub
		return name != null && !name.isEmpty();
	}

	@Override
	protected boolean isValidPasswd(String passwd) {
		// TODO Auto-generated method stub
		return passwd != null && !passwd.isEmpty();
	}
	
	private void log(String msg) {
		if(!logEnabled)
			throw new NullPointerException();
		else
			System.out.println(System.currentTimeMillis() + "==> " + msg);
	}
	
	private boolean logEnabled;
	
	private Map<ArrayList<Byte>, ArrayList<Byte>> data;
	private Map<ArrayList<Byte>, ArrayList<Byte>> users;
	private List<ArrayList<Byte>> loggedin;
	
	private Encryption e;
	private int dbload = 0;

}
