package task.db2;

import org.junit.Ignore;
import org.junit.Test;

import com.google.java.contract.InvariantError;
import com.google.java.contract.PreconditionError;
import com.google.java.contract.PostconditionError;

import task.db2.Database;
import task.db2.PlainDB;

public class DBClient {
	
	@Ignore
    @Test(expected = PreconditionError.class)
    public void testDuplicateRegister() {
        Database db = new PlainDB();
        db.register("user1", "pass1");
        db.register("user1", "pass2");
    }
    
	@Ignore
    @Test(expected = PreconditionError.class)
    public void testDuplicateLogin() {
        Database db = new PlainDB();
        db.register("user1", "pass1");
        db.login("user1", "pass1");
        db.login("user1", "pass2");
    }
    
	@Ignore
    @Test(expected = PreconditionError.class)
    public void testInvalidLogin() {
        Database db = new PlainDB();
        db.register("user1", "pass1");
        db.login("user2", "pass2");
    }
    
	@Ignore
    @Test(expected = PreconditionError.class)
    public void testInvalidDataRead() {
        Database db = new PlainDB();
        db.register("user1", "pass1");
        db.login("user1", "pass1");
        db.addData("user1", 1, 10);
        db.readData("user1", 2);
    }
    
	@Ignore
    @Test(expected = PreconditionError.class)
    public void testPasswordIncorrect() {
        Database db = new PlainDB();
        db.register("user1", "pass1");
        db.login("user1", "pass2");
    }
    
	@Ignore
    @Test(expected = PreconditionError.class)
    public void testDuplicateData() {
        Database db = new PlainDB();
        db.register("user1", "pass1");
        db.login("user1", "pass2");
        db.addData("user1", 1, 10);
        db.addData("user1", 1, 10);
    }
	
	
	@Ignore
    @Test//(expected = PostconditionError.class)
    public void testPostCondition() {
    	Database.init();
        Database dbp = new PlainDB();
        
        dbp.register("user1", "pass1");
        dbp.login("user1", "pass1");
        dbp.addData("user1", 10, 100);
        dbp.addData("user1", 20, 200);
        
        System.out.println(dbp.readData("user1", 20));

    }
	
	@Ignore
    @Test//(expected = PreconditionError.class)
    public void testEmptyData() {
    	Database.init();
        Database dbs = new SecureDB(true);
        dbs.register("", "pass1");
    }
    
	@Ignore
    @Test//(expected = InvariantError.class)
    public void testNetworkLoad() {
    	Database.init();
        Database dbs = new SecureDB(true);
        Database dbp = new PlainDB();
        
        dbs.register("user1", "pass1");
        dbp.register("user2", "pass2");
        dbp.register("user3", "pass3");
        dbp.register("user4", "pass4");
        
        
        dbs.login("user1", "pass1");
        dbp.login("user2", "pass2");
        dbp.login("user3", "pass3");
        dbp.login("user4", "pass4");
    }
    
	@Ignore
    @Test//(expected = InvariantError.class)
    public void testSecureDBLoad() {
    	Database.init();
        Database dbs = new SecureDB(true);
        
        dbs.register("user1", "pass1");
        dbs.register("user2", "pass2");
        dbs.register("user3", "pass3");
        
        
        dbs.login("user1", "pass1");
        dbs.login("user2", "pass2");
        dbs.login("user3", "pass3");
    }
    
	@Ignore
    @Test//(expected = PostconditionError.class)
    public void testEncryption() {
    	Database.init();
        Database dbs = new SecureDB(true);
        
        dbs.register("user1", "pass1");
        dbs.login("user1", "pass1");
        dbs.addData("user1", 10, 100);
        System.out.println(dbs.readData("user1", 10));
    }
    
	@Ignore
    @Test//(expected = PreconditionError.class)
    public void testLogData() {
    	Database.init();
    	Database dbs = new SecureDB(false);
    	dbs.register("user1", "pass1");
        dbs.login("user1", "pass1");
    }
    
    
}
